@@ -6,13 +6,15 @@ use DBI;
 use File::Temp;
 use Mojo::SQLite::Database;
 use Mojo::SQLite::Migrations;
-use Mojo::URL;
 use Scalar::Util 'weaken';
+use URI;
+use URI::QueryParam;
+use URI::file;
 
-our $VERSION = '0.007';
+our $VERSION = '0.008';
 
 has dsn => sub {
-  my $uri = Mojo::URL->new->scheme('file')->path(shift->_tempfile);
+  my $uri = _uri_from_path(shift->_tempfile_path);
   return "dbi:SQLite:uri=$uri";
 };
 has max_connections => 5;
@@ -41,26 +43,28 @@ sub db {
   return Mojo::SQLite::Database->new(dbh => $self->_dequeue, sqlite => $self);
 }
 
+sub from_filename { shift->from_string(URI::file->new(shift)) }
+
 sub from_string {
   my ($self, $str) = @_;
 
-  # Protocol
+  # Parse URI
   return $self unless $str;
-  my $url = Mojo::URL->new($str);
-  my $protocol = $url->protocol;
-  my $host = $url->host // '';
-  my $path = $url->path;
-  my $options = $url->query->to_hash;
+  my $uri = URI->new($str);
+  my $scheme = $uri->scheme // '';
+  my $host = $scheme eq 'file' ? ($uri->host // '') : '';
   croak qq{Invalid SQLite connection string "$str"}
-    unless ($protocol eq '' or $protocol eq 'file')
+    unless ($scheme eq '' or $scheme eq 'file')
     and ($host eq '' or $host eq 'localhost');
-
+  
   # Database file
-  $path = $self->_tempfile if $path eq ':temp:';
-  my $uri = Mojo::URL->new->scheme('file')->path($path);
-  my $dsn = "dbi:SQLite:uri=$uri";
+  my $path = $uri->path;
+  $path = $self->_tempfile_path if $path eq ':temp:';
+  my $new_uri = _uri_from_path($path);
+  my $dsn = "dbi:SQLite:uri=$new_uri";
 
   # Options
+  my $options = $uri->query_form_hash;
   @{$self->options}{keys %$options} = values %$options;
 
   return $self->dsn($dsn);
@@ -85,12 +89,20 @@ sub _enqueue {
   shift @$queue while @$queue > $self->max_connections;
 }
 
-sub _tempfile {
+sub _tempfile_path {
   my $self = shift;
   $self->{tempfile} = File::Temp->new(EXLOCK => 0);
-  return $self->{tempfile}->filename;
+  return URI::file->new($self->{tempfile}->filename)->path;
 };
 
+sub _uri_from_path {
+  my $path = shift;
+  my $uri = URI->new;
+  $uri->scheme('file');
+  $uri->path($path);
+  return $uri;
+}
+
 1;
 
 =head1 NAME
@@ -100,10 +112,9 @@ Mojo::SQLite - A tiny Mojolicious wrapper for SQLite
 =head1 SYNOPSIS
 
   use Mojo::SQLite;
-  use Mojo::URL;
 
   # Create a table
-  my $sql = Mojo::SQLite->new(Mojo::URL->new->scheme('file')->path($filename));
+  my $sql = Mojo::SQLite->new->from_filename($filename);
   $sql->db->query('create table names (id integer primary key autoincrement, name text)');
 
   # Insert a few rows
@@ -146,10 +157,9 @@ gracefully by holding on to them only for short amounts of time.
 
   use Mojolicious::Lite;
   use Mojo::SQLite;
-  use Mojo::URL;
 
   helper sqlite =>
-    sub { state $sql = Mojo::SQLite->new(Mojo::URL->new->path(shift->config('sqlite_filename')) };
+    sub { state $sql = Mojo::SQLite->new->from_filename(shift->config('sqlite_filename')) };
 
   get '/' => sub {
     my $c  = shift;
@@ -173,6 +183,10 @@ All cached database handles will be reset automatically if a new process has
 been forked, this allows multiple processes to share the same L<Mojo::SQLite>
 object safely.
 
+Any database errors will throw an exception as C<RaiseError> is automatically
+enabled, so use C<eval> or L<Try::Tiny> to catch them. This makes transactions
+with L<Mojo::SQLite::Database/"begin"> easy.
+
 While passing a file path of C<:memory:> (or a custom L</"dsn"> with
 C<mode=memory>) will create a temporary database, in-memory databases cannot be
 shared between connections, so subsequent calls to L</"db"> may return
@@ -230,11 +244,13 @@ more easily.
   my $options = $sql->options;
   $sql        = $sql->options({AutoCommit => 1, RaiseError => 1});
 
-Options for database handles, defaults to activating C<AutoCommit>,
-C<AutoInactiveDestroy> as well as C<RaiseError> and deactivating C<PrintError>.
-Note that C<AutoCommit> and C<RaiseError> are considered mandatory, so
-deactivating them would be very dangerous. L<DBD::SQLite> specific option
-C<sqlite_unicode> is also set by default.
+Options for database handles, defaults to activating C<sqlite_unicode>,
+C<AutoCommit>, C<AutoInactiveDestroy> as well as C<RaiseError> and deactivating
+C<PrintError>. Note that C<AutoCommit> and C<RaiseError> are considered
+mandatory, so deactivating them would be very dangerous. See
+L<DBI/"ATTRIBUTES COMMON TO ALL HANDLES"> and
+L<DBD::SQLite/"DRIVER PRIVATE ATTRIBUTES"> for more information on available
+options.
 
 =head1 METHODS
 
@@ -254,14 +270,39 @@ gracefully by holding on to it only for short amounts of time.
   say $sql->db->query('select * from accounts')
     ->hashes->reduce(sub { $a->{money} + $b->{money} });
 
+=head2 from_filename
+
+  $sql = $sql->from_filename('C:\\Documents and Settings\\foo & bar.db');
+
+Parse database filename directly. Unlike L</"from_string">, the filename is
+parsed as a local filename and not a URI, and L</"options"> must be specified
+separately.
+
+  # Absolute filename
+  $sql->from_filename('/home/fred/data.db');
+
+  # Relative to current directory
+  $sql->from_filename('data.db');
+
+  # Temporary file database (default)
+  $sql->from_filename(':temp:');
+
+  # In-memory temporary database (single connection only)
+  my $db = $sql->from_filename(':memory:')->db;
+
+  # Additional options
+  $sql->from_filename($filename)->options({PrintError => 1});
+
 =head2 from_string
 
   $sql = $sql->from_string('file:test.db');
 
 Parse configuration from connection string. Connection strings are parsed as
 URIs, so you should construct them using a module like L<Mojo::URL> or
-L<URI::file>. The scheme and hostname are optional, but if specified must be
-C<file> and C<localhost> respectively.
+L<URI::file>. L<URI::file> is recommended for portability on non-Unix-like
+systems. The scheme and hostname are optional, but if specified must be C<file>
+and C<localhost> respectively. If the connection string has a query string, it
+will be parsed and applied to L</"options">.
 
   # Absolute filename
   $sql->from_string('file:///home/fred/data.db');
@@ -276,7 +317,7 @@ C<file> and C<localhost> respectively.
   $sql->from_string('data.db');
 
   # Connection string must be a valid URI
-  $sql->from_string(Mojo::URL->new->scheme('file')->path($filename));
+  $sql->from_string(Mojo::URL->new->path($unix_filename));
   $sql->from_string(URI::file->new($filename));
 
   # Temporary file database (default)
@@ -287,19 +328,26 @@ C<file> and C<localhost> respectively.
 
   # Additional options
   $sql->from_string('data.db?PrintError=1&sqlite_allow_multiple_statements=1');
-  $sql->from_string(Mojo::URL->new->scheme('file')->path($filename)->query(PrintError => 1));
+  
+  my $uri = URI::file->new($filename);
+  $uri->query_form_hash({PrintError => 1, sqlite_allow_multiple_statements => 1});
+  $sql->from_string($uri);
 
 =head2 new
 
   my $sql = Mojo::SQLite->new;
-  my $sql = Mojo::SQLite->new('test.db');
+  my $sql = Mojo::SQLite->new('file:test.db');
 
 Construct a new L<Mojo::SQLite> object and parse connection string with
 L</"from_string"> if necessary.
 
   # Customize configuration further
+  my $sql = Mojo::SQLite->new->dsn('dbi:SQLite:dbname=test.db');
   my $sql = Mojo::SQLite->new->dsn('dbi:SQLite:uri=file:test.db?mode=memory');
 
+  # Pass filename directly
+  my $sql = Mojo::SQLite->new->from_filename($filename);
+
 =head1 REFERENCE
 
 This is the class hierarchy of the L<Mojo::SQLite> distribution.
